/*

BBC Radio widget by Hawkman. http://www.phantomgorilla.com/
All suggestions and bug reports to hawkman@phantomgorilla.com please

*******************************************************************************

LICENCE

Free to use, free to look at and learn from, free to small chunks of code from for use in your own projects (eg. reproduction of a couple of functions). You are also free to modify it as you see fit, for your own personal use only.

However, you may NOT redistribute this widget, or a port or modified version of it, without obtaining permission from the author (via email, hawkman@phantomgorilla.com) and including credit to him in the finished project (a small notice in the widget's javascript/html files is all that's necessary; nothing visible on the widget).

Small scale distribution of this widget or a modified version of it, such as that between family members or friends, is specifically allowed despite these restrictions.

This software is provided as-is, with no guarantee. The author can accept no responsibility for any consequences of its use.

*******************************************************************************

ABOUT THE WIDGET

- Basic Overview

This widget uses Real Player for the audio. Preferences are written upon the selection of an item in a pop-up menu, then this is compared to a list of stations to find the stream url. A function then writes html to the front of the widget to embed the realplayer plugin with the correct url, and changes the image source to reflect which station is playing.

- Credits

Basic Javascript routines taken out of Apple's sample Goodbye World widget. A small section of crash-preventing code is taken from Duncan Ponting's radio widget (http://www.bbc.co.uk/radio/prototypes/listenlive/) at his suggestion - cheers. Also taken from that widget is the idea to crawl the broswer plugins to find RealPlayer. Version string .plist lookup taken from Joshua Emmons' blank foo.wdgt (http://www.skia.net/). Study of his example AppleAnimator use was also extremely helpful. Some button-like routines adapted from Apple's GenericButton class.

Additional skills were obtained by studying Apple's sample files and the Mac OS X bundled widgets, however no actual code was used from these sources other than that mentioned already.

******************************************************************************/



/*******************************/
// GLOBAL VARIABLES & SETTINGS
/*******************************/

var debug = false;

// Global variable for the status animation
var statusAnimator = null;
var statusTimeout = null;
// and one for the drawer animation
var drawerAnimator = null;

var volumeCounter;

// sets up a variable to store the XML from checkForUpdate(), also to let
// us know if a request is currently in progress
var externalXmlRequest = null;

// variable to store the current version number from Info.plist; only set on
// load, but referred to on multiple occasions
var currentVersion;

// holds the version number of the latest version available; initially taken
// from preferences (if there), it's updated every day by checkForUpdate()
var latestVersion = new Object();
latestVersion.release = (window.widget ? widget.preferenceForKey("latestversion") : "");
latestVersion.release = splitVersion(latestVersion.release);
latestVersion.betaRelease = (window.widget ? widget.preferenceForKey("LatestBeta") : "");
latestVersion.betaRelease = splitVersion(latestVersion.betaRelease);
var updateUrl = (window.widget ? widget.preferenceForKey("UpdateUrl") : null);
// get the last time we checked for an update
var then = (window.widget ? widget.preferenceForKey("lastupdatecheck") : 0);

// all involved with the button handlers such as theMouseDownHandler //
// this one stores whether the mouse button is down
var theCurrentMouseDownButton = null;
// the button which was clicked
var wasClickedElement = null;
// the element to swap images into
var isElementToChange = null;
// the starting position of the left side of the volume knob
var knobStartLeft = null;
// the cursor's starting x-coordinate; used with the slider
var cursorStartX = null;
// left boundary of the slider
var leftBound = 75;
// right boundary of the slider
var rightBound = 108;

// to remember the selection before showing the rear
var preferenceBefore = null;

// used to temporarily store event settings if we use the showStatus() function,
// as we can't pass these directly from it to hideStatus() via setTimeout().
var globalStatusEvent;

// the xmlhttprequest object we use later
var xmlhttp;
var testInProg = null;

// flash flag
var UseFlash = false;
var flashTimer = new Object();
flashTimer.seconds = 0;
flashTimer.trigger;

// flags for hidden plist keys
var useHack = true;
var disableTests = false;

var justShownRestartMessage = false;

// sets our onshow function to execute at the right times
if (window.widget) {
	widget.onshow = onShow;
	widget.onhide = onHide;
    widget.onremove = onRemove;
}

var stationList = {
	"radio1": {
		"name": "Radio 1",
		"nickname": "radio1",
		"logo": "r1.png",
		"realplayer": "http://www.bbc.co.uk/radio/listen/live/r1.rpm",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_one",
		"api": "BBCROne",
		"coverage": "national"
	},
	"radio2": {
		"name": "Radio 2",
		"nickname": "radio2",
		"logo": "r2.png",
		"realplayer": "http://www.bbc.co.uk/radio/listen/live/r2.rpm",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_two",
		"api": "BBCRTwo",
		"coverage": "national"
	},
	"radio3": {
		"name": "Radio 3",
		"nickname": "radio3",
		"logo": "r3.png",
		"realplayer": "http://www.bbc.co.uk/radio/listen/live/r3.rpm",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_three",
		"api": "BBCRThree",
		"coverage": "national"
	},
	"radio4": {
		"name": "Radio 4 FM",
		"nickname": "radio4",
		"logo": "r4.png",
		"realplayer": "http://www.bbc.co.uk/radio/listen/live/r4.rpm",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_fourfm",
		"api": "BBCRFour",
		"extra": "FM",
		"coverage": "national"
	},
	"radio4LW": {
		"name": "Radio 4 LW",
		"nickname": "radio4",
		"logo": "r4.png",
		"realplayer": "http://www.bbc.co.uk/radio/listen/live/r4lw.rpm",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_fourlw",
		"extra": "LW",
		"coverage": "national"
	},
	"fivelive": {
		"name": "Radio 5 Live",
		"nickname": "fivelive",
		"logo": "r5.png",
		"realplayer": "http://www.bbc.co.uk/radio/listen/live/r5l.rpm",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_five_live",
		"api": "BBCRFiveL",
		"coverage": "national"
	},
	"6music": {
		"name": "BBC 6 Music",
		"nickname": "6music",
		"logo": "6music.png",
		"realplayer": "http://www.bbc.co.uk/radio/listen/live/r6.rpm",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_6music",
		"api": "BBCSixMU",
		"coverage": "national"
	},
	"radio7": {
		"name": "BBC 7",
		"nickname": "radio7",
		"logo": "bbc7.png",
		"realplayer": "http://www.bbc.co.uk/radio/listen/live/r7.rpm",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_7",
		"api": "BBCSeven",
		"coverage": "national"
	},
	"1xtra": {
		"name": "1Xtra",
		"nickname": "1xtra",
		"logo": "1xtra.png",
		"realplayer": "http://www.bbc.co.uk/radio/listen/live/r1x.rpm",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_1xtra",
		"api": "OneXtra",
		"coverage": "national"
	},
	"5livesportsextra": {
		"name": "5L Sports Extra",
		"nickname": "5livesportsextra",
		"logo": "5livextra.png",
		"realplayer": "http://www.bbc.co.uk/radio/listen/live/r5lsp.rpm",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_five_live_sports_extra",
		"api": "BBCRFiveX",
		"coverage": "national"
	},
	"asiannetwork": {
		"name": "Asian Network",
		"nickname": "asiannetwork",
		"logo": "asian.png",
		"realplayer": "http://www.bbc.co.uk/radio/listen/live/ran.rpm",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_asian_network",
		"api": "BBCAsian",
		"coverage": "national"
	},
	"worldservice": {
		"name": "World Service",
		"nickname": "worldservice",
		"logo": "ws.png",
		"realplayer": "http://www.bbc.co.uk/worldservice/meta/tx/nb/live_infent_au_nb.ram",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_world_service",
		"api": "BBCWrld",
		"coverage": "national"
	},
	"radioscotland": {
		"name": "Radio Scotland",
		"nickname": "radioscotland",
		"logo": "scotland.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_scotland_fm",
		"coverage": "regional"
	},
	"radiowales": {
		"name": "Radio Wales",
		"nickname": "radiowales",
		"logo": "wales.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_wales_fm",
		"coverage": "regional"
	},
	"radioulster": {
		"name": "Radio Ulster",
		"nickname": "radioulster",
		"logo": "ulster.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_ulster",
		"coverage": "regional"
	},
	"radiocymru": {
		"name": "Radio Cymru",
		"nickname": "radiocymru",
		"logo": "cymru.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_cymru",
		"coverage": "regional"
	},
	"radiofoyle": {
		"name": "Radio Foyle",
		"nickname": "radiofoyle",
		"logo": "foyle.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_foyle",
		"coverage": "regional"
	},
	"radionangaidheal": {
		"name": "Radio nan G&#x00E0;idheal",
		"nickname": "radionangaidheal",
		"logo": "ng.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_nan_gaidheal",
		"coverage": "regional"
	},
	"berkshire": {
		"name": "Berkshire",
		"nickname": "berkshire",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_berkshire",
		"coverage": "local"
	},
	"bristol": {
		"name": "Bristol",
		"nickname": "bristol",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_bristol",
		"coverage": "local"
	},
	"cambridgeshire": {
		"name": "Cambridgeshire",
		"nickname": "cambridgeshire",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_cambridge",
		"coverage": "local"
	},
	"cornwall": {
		"name": "Cornwall",
		"nickname": "cornwall",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_cornwall",
		"coverage": "local"
	},
	"coventry": {
		"name": "Coventry & Warwickshire",
		"nickname": "coventry",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_coventry_warwickshire",
		"shortname": "Coventry & Warks",
		"coverage": "local"
	},
	"cumbria": {
		"name": "Cumbria",
		"nickname": "cumbria",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_cumbria",
		"coverage": "local"
	},
	"derby": {
		"name": "Derby",
		"nickname": "derby",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_derby",
		"coverage": "local"
	},
	"devon": {
		"name": "Devon",
		"nickname": "devon",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_devon",
		"coverage": "local"
	},
	"essex": {
		"name": "Essex",
		"nickname": "essex",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_essex",
		"coverage": "local"
	},
	"gloucestershire": {
		"name": "Gloucestershire",
		"nickname": "gloucestershire",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_gloucestershire",
		"coverage": "local"
	},
	"guernsey": {
		"name": "Guernsey",
		"nickname": "guernsey",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_guernsey",
		"coverage": "local"
	},
	"herefordandworcester": {
		"name": "Hereford & Worcester",
		"nickname": "herefordandworcester",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_hereford_worcester",
		"shortname": "H’ford & Worcester",
		"coverage": "local"
	},
	"humberside": {
		"name": "Humberside",
		"nickname": "humberside",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_humberside",
		"coverage": "local"
	},
	"jersey": {
		"name": "Jersey",
		"nickname": "jersey",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_jersey",
		"coverage": "local"
	},
	"kent": {
		"name": "Kent",
		"nickname": "kent",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_kent",
		"coverage": "local"
	},
	"lancashire": {
		"name": "Lancashire",
		"nickname": "lancashire",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_lancashire",
		"coverage": "local"
	},
	"leeds": {
		"name": "Leeds",
		"nickname": "leeds",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_leeds",
		"coverage": "local"
	},
	"leicester": {
		"name": "Leicester",
		"nickname": "leicester",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_leicester",
		"coverage": "local"
	},
	"lincolnshire": {
		"name": "Lincolnshire",
		"nickname": "lincolnshire",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_lincolnshire",
		"coverage": "local"
	},
	"london": {
		"name": "London",
		"nickname": "london",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_london",
		"coverage": "local"
	},
	"manchester": {
		"name": "Manchester",
		"nickname": "manchester",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_manchester",
		"coverage": "local"
	},
	"merseyside": {
		"name": "Merseyside",
		"nickname": "merseyside",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_merseyside",
		"coverage": "local"
	},
	"newcastle": {
		"name": "Newcastle",
		"nickname": "newcastle",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_newcastle",
		"coverage": "local"
	},
	"norfolk": {
		"name": "Norfolk",
		"nickname": "norfolk",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_norfolk",
		"coverage": "local"
	},
	"northampton": {
		"name": "Northampton",
		"nickname": "northampton",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_northampton",
		"coverage": "local"
	},
	"nottingham": {
		"name": "Nottingham",
		"nickname": "nottingham",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_nottingham",
		"coverage": "local"
	},
	"oxford": {
		"name": "Oxford",
		"nickname": "oxford",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_oxford",
		"coverage": "local"
	},
	"sheffield": {
		"name": "Sheffield",
		"nickname": "sheffield",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_sheffield",
		"coverage": "local"
	},
	"shropshire": {
		"name": "Shropshire",
		"nickname": "shropshire",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_shropshire",
		"coverage": "local"
	},
	"solent": {
		"name": "Solent",
		"nickname": "solent",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_solent",
		"coverage": "local"
	},
	"somerset": {
		"name": "Somerset",
		"nickname": "somerset",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_somerset_sound",
		"coverage": "local"
	},
	"stoke": {
		"name": "Stoke",
		"nickname": "stoke",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_stoke",
		"coverage": "local"
	},
	"suffolk": {
		"name": "Suffolk",
		"nickname": "suffolk",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_suffolk",
		"coverage": "local"
	},
	"surrey": {
		"name": "Surrey",
		"nickname": "surrey",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_surrey",
		"coverage": "local"
	},
	"sussex": {
		"name": "Sussex",
		"nickname": "sussex",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_sussex",
		"coverage": "local"
	},
	"tees": {
		"name": "Tees",
		"nickname": "tees",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_tees",
		"coverage": "local"
	},
	"threecounties": {
		"name": "Three Counties",
		"nickname": "threecounties",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_three_counties_radio",
		"coverage": "local"
	},
	"wiltshire": {
		"name": "Wiltshire",
		"nickname": "wiltshire",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_wiltshire",
		"coverage": "local"
	},
	"wm": {
		"name": "WM",
		"nickname": "wm",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_wm",
		"coverage": "local"
	},
	"york": {
		"name": "York",
		"nickname": "york",
		"logo": "local.png",
		"flash": "http://www.bbc.co.uk/iplayer/playlist/bbc_radio_york",
		"coverage": "local"
	}
}

var currentStation = stationList.radio1;


myScrollwheel = function(event)
{
    var area = document.getElementById("scrollArea").object;
	if (event.wheelDeltaY) {
        var deltaScroll = event.wheelDeltaY / 120 * area.singlepressScrollPixels;
    } else if (event.wheelDeltaX) {
        var deltaScroll = 0;
    } else {
        var deltaScroll = event.wheelDelta / 120 * area.singlepressScrollPixels;
    }
    // make sure we scroll, however little we've moved
    if (deltaScroll<1 && deltaScroll>0) deltaScroll=1;
    if (deltaScroll>-1 && deltaScroll<0) deltaScroll=-1;

	area.verticalScrollTo(area.content.scrollTop - deltaScroll);

	event.stopPropagation();
	event.preventDefault();
}


/*****************************************************/
// SETUP FUNCTIONS, SAVING AND RETRIEVING PREFERENCES
/*****************************************************/


// onLoad() is run when the body loads.  It checks to see if there is a
// preference for this widget and if so, applies the preference to the widget.
function onLoad()
{
	if(window.widget) { // always check to make sure that you are running in Dashboard
        // draw the info button, using AppleClasses
        new AppleInfoButton (document.getElementById('info'), 
	                     document.getElementById('front'), 
	                     'black', 'black', showPrefs);
		// a clone of Apple's info button with minor modifications
        new drawerButton (document.getElementById('drawerbutton'), 
	                     document.getElementById('front'), 
	                     'black', doDrawerAction);

        // do some upgrade-related cleanup
        var lastUsedVersion = widget.preferenceForKey("last_used_version");
        if (!lastUsedVersion || lastUsedVersion.length <=0) {
            var lastUsedVersion = widget.preferenceForKey("LastUsedVersion");
        }
        lastUsedVersion = splitVersion(lastUsedVersion);
        if (widget.preferenceForKey("WaitingForRestart")) {
            widget.setPreferenceForKey(null,"WaitingForRestart");
            //if (lastUsedVersion==currentVersion) showStatus(null,'Thanks!<br>BBC Radio Widget has<br>now been successfully<br>upgraded to v'+currentVersion+'.',8);
        }
        // calls fillVersion() to fill in the version number on the rear
        fillVersion();
        // upgrade if needed, else load current prefs
        if (lastUsedVersion.major==0 || checkIfVersionHigher(lastUsedVersion, currentVersion)) {
            doUpgrade(lastUsedVersion);
        } else {
            // loads the current station from preferences and looks it up
            // first tries a key for this instance
            var temp_currentStation = widget.preferenceForKey(makeKey("CurrentStation"));
            // if there's nothing for this instance
            if (!temp_currentStation || temp_currentStation.length <= 0) {
                // loads default key for new instances, and saves it as this instance's station
                temp_currentStation = widget.preferenceForKey("CurrentStation");
                widget.setPreferenceForKey(temp_currentStation, makeKey("CurrentStation"));
            }
            if (debug) alert("Versions match. No upgrade needed. Loading current station: "+temp_currentStation);
            // check it makes sense and load the station's info
            if (temp_currentStation && temp_currentStation.length > 0) {
                if (temp_currentStation in stationList) {
                    currentStation = stationList[temp_currentStation];
                } else {
                    // saved key doesn't make sense, so stick with what we set when defining variables.
                    // save for this instance and as default for new instances
                    widget.setPreferenceForKey(currentStation.nickname,"CurrentStation");
                    widget.setPreferenceForKey(currentStation.nickname,makeKey("CurrentStation"));
                }
            }
        }
        // rebuilds the station selection menu
        buildStationSelect();
        // gets and shows the correct station image
        changeImage();
		// changes the image on the schedule drawer button
		updateDrawerButton();
        
        // flash use
        checkForFlash();

        // sets the volume slider to the correct position
        restoreVolume();
        
        // draw the Done button for the back - again, AppleClasses
        new AppleGlassButton(document.getElementById('done'), 'Done', hidePrefs);
        new AppleGlassButton(document.getElementById('done2'), 'Done', hidePrefs2);
        
        // make schedule scroll less dramatically fast
        var area = document.getElementById("scrollArea").object;
        area.content.removeEventListener("mousewheel", area._mousewheelScrollHandler, true);
        area.content.addEventListener("mousewheel", myScrollwheel, true);

        // decides if we should use the RP11 hack
        checkForHackUse();
        // lets us disable connection/plugin tests
        checkForTestDisable();

		// refreshes the schedule
		refreshSchedule();

	}
    checkForUpdate();
}


// delete temporary preferences, stop timers
function onRemove() {
    widget.setPreferenceForKey(null,makeKey("CurrentStation"));
    widget.setPreferenceForKey(null,makeKey("Volume"));
    clearTimeout(volumeCounter);
    clearTimeout(statusTimeout);
    clearTimeout(testInProg);
    clearInterval(flashTimer.trigger);
    clearTimeout(schedInProg);
    clearTimeout(scheduleTimer);
    clearInterval(progressIndicatorTimer);
}


function buildStationSelect() {
    var menu = document.getElementById("station");
    menu.innerHTML="";

    var national = document.createElement("optgroup");
    national.label = "National";
    var regional = document.createElement("optgroup");
    regional.label = "Regional";
    var local = document.createElement("optgroup");
    local.label = "Local";
    
    var stationToSelect = currentStation.nickname;
    if (currentStation.nickname=="radio4") {
        if (currentStation.extra=="LW") {
            stationToSelect = "radio4LW";
        }
    }

    for (key in stationList) {
        var station = (stationList[key]);
        var newStation = document.createElement("option");
        newStation.value = key;
        newStation.innerHTML = station.name;
        if (key==stationToSelect) newStation.selected = true;
        if (station.coverage=="national") {
            national.appendChild(newStation);
        } else if (station.coverage=="regional") {
            regional.appendChild(newStation);
        } else {
            local.appendChild(newStation);
        }
    }
    menu.appendChild(national);
    menu.appendChild(regional);
    menu.appendChild(local);
}


// onShow() runs whenever Dashboard loads. We use it to refresh the
// schedule information in case it's out of date.
function onShow()
{
	// refresh the schedule & force a scroll to the programme
	refreshSchedule("forceScroll");
    // check play state
    var plugin = document.getElementById("rp");
    var radioHtml = document.getElementById("radio").innerHTML;
    // if plugin is paused (accidentally), start playback again
    if (plugin && plugin.GetPlayState()==4) {
        pauseClicked();
        playClicked();
    } else if (radioHtml=="") {
        // cleanup just in case
        // causes problems with RP11 hack
        //pauseClicked();
    }
    if (widget.preferenceForKey("WaitingForRestart")) {
        if (!justShownRestartMessage) showStatus(null,'Widget upgraded! Please reload (&#x2318;R) the widget or click this message to restart Dashboard.',30,function(){widget.system("killall Dock; sleep 3; [[ -e /Applications/Dashboard.app ]] && open /Applications/Dashboard.app", killDashboardHandler);});
        justShownRestartMessage = false;
    }
    // check to see what the latest version is; at intervals of a day, this
    // function will check online, otherwise it uses a local cache of the last
    // known value.
    checkForUpdate();
}


// onHide() runs whenever Dashboard is hidden. It stops the timer
// calling refreshSchedule() and refreshing the scedule view.
function onHide()
{
	clearTimeout(scheduleTimer);
	scheduleTimer = null;
    var plugin = document.getElementById("rp");
    // RealPlayer 11 workaround
    // only applies if version is 11+
    // can be disabled by flag (set by OS version/plist key, default on)
    if (useHack) {
        if (plugin) {
            var rpVer = plugin.GetVersionInfo().split(".");
            if (rpVer.length>=1) rpVer = parseInt(rpVer[0]);
            if (rpVer>=11) {
                if (plugin.CanStop()) {
                    // stops audio when widget is hidden; OnPlayStateChange
                    // callback will restart it!
                    plugin.DoStop();
                } else if (document.getElementById("radio").innerHTML!="") {
                    setTimeout("pauseClicked(); playClicked();", 1500);
                }
            }
        // if there's no plugin element, we can't tell the version
        // check if the "radio" holder has html (which means
        // we're not paused) and if so, apply a RP11 workaround
        // just to be safe.
        } else if (UseFlash && document.getElementById("radio").innerHTML!="") {
        } else if (document.getElementById("radio").innerHTML!="") {
            setTimeout("pauseClicked(); playClicked();", 1200);
        }
    }
    // workaround for an audio loss that could occur when leaving Dashboard
    // before before Flash is placed
    if (flashTimer.trigger != null && flashTimer.seconds <= 4) {
        changeStream();
        alert("did it");
    }
}


// called from onLoad() to fill in the author & version information on the rear;
// it grabs them from Info.plist
function fillVersion()
{
    // start a new XMLHttpRequest
    var internalXmlRequest = new XMLHttpRequest();
    // setup the destination of the request (Info.plist)
    internalXmlRequest.open("GET", "Info.plist", false);
    // send nothing
    internalXmlRequest.send(null);
    // get all the "key" tags, set them to an array(?)
    var keys = internalXmlRequest.responseXML.getElementsByTagName("key");
    // set the variable to 0.0, so we have a value just in case this fails
    var temp_currentVersion = 0;
    // set up a loop, to go through every key tag
    for(i=0; i<keys.length; i++)
    {
        // if we hit a match for CFBundleVersion in the data of the "first
        // child" of any of the keys (XML navigation terminology is rather
        // confusing)
        if("CFBundleVersion" == keys[i].firstChild.data)
        {
            // if we hit CFBundleVersion, we know where the actual version
            // number will be inrelation to it (assuming the .plist is valid)
            // so we can navigate to it - again, confusing-looking, no?
            // Anyway, we set it to the global varible, so we can refer to it
            // in other functions
            temp_currentVersion = keys[i].nextSibling.nextSibling.firstChild.data;
            // break the loop, we got a match so we're done
            break;
        }
    }
    // uses the newly-set variable to fill in the version number, along with
    // the rest of the information, into the span with id="author"
	document.getElementById("author").innerHTML = "BBC Radio v"+temp_currentVersion+" by Hawkman\nSupported by <a href=\"javascript:widget.openURL('http://backstage.bbc.co.uk')\">BBC Backstage</a>";
    currentVersion = splitVersion(temp_currentVersion);
    if (debug) alert("Constructed version number: "+versionDisplayString(currentVersion));
}


// constructs an object containing a complex version string,
// including an integer beta value if provided.
// takes a string, "X.xxbX" as an argument; X can be many digits
// long, but xx will be truncated to 2 digits.
// so eg. 4.9 (=4.90) > 4.11 > 4.108 (=4.10) > 4.02
function splitVersion(verString) {
    var ver = new Object();
    verString += "";
    if (verString.indexOf("b")!=-1) {
        ver.beta = parseInt(verString.split("b")[1]);
        if (isNaN(ver.beta)) ver.beta = null;
        verString = verString.substring(0,verString.indexOf("b"));
    } else {
        ver.beta = null;
    }
        
    if (verString.indexOf(".")!=-1) {
        var temp_minor = verString.split(".")[1];
        if (temp_minor.length==1) { temp_minor+="0"; }
        else if (temp_minor.length>2) { temp_minor=temp_minor.substr(0,2); }
        ver.minor = parseInt(temp_minor);
        if (isNaN(ver.minor)) ver.minor = 0;
        verString = verString.substring(0,verString.indexOf("."));
    } else {
        ver.minor = 0;
    }
    
    ver.major = parseInt(verString);
    if (isNaN(ver.major)) ver.major = 0;

    if (debug) alert("Version was created: "+versionDisplayString(ver));
    return ver;
}


function checkIfVersionHigher(baseline, contender) {
    // sanity check; we should never create versions
    // which fail, but just in case
    if (!baseline.major) baseline.major = 0;
    if (!baseline.minor) baseline.major = 0;
    if (!contender.major) contender.major = 0;
    if (!contender.minor) contender.major = 0;

    if (debug) alert("Comparing versions: "+versionDisplayString(baseline)+" and "+versionDisplayString(contender));
    
    if (contender.major>baseline.major) {
        return true;
    } else if (contender.major<baseline.major) {
        return false;
    }
    
    if (contender.minor>baseline.minor) {
        return true;
    } else if (contender.minor<baseline.minor) {
        return false;
    }

    // if we've reached this point, numbers match so
    // we're into checking if only one is a beta (it's older),
    // or comparing beta revisions
    if (baseline.beta && !contender.beta) {
        return true;
    } else if (!baseline.beta && contender.beta) {
        return false;
    }
    if (contender.beta>baseline.beta) {
        return true;
    } else if (contender.beta<baseline.beta) {
        return false;
    }
    
    return false;
}


// checks to see what the latest version is. Every two days it looks online at
// my website, where the latest version will be published; in between it uses
// a local cache of the last known value instead.
function checkForUpdate()
{
    // get the current date
    var now = (new Date).getTime();
    // if we haven't checked for an update, or if it's been a day. Used to only
    // allow this if an update hadn't been detected previously; however, this
    // doesn't allow for human error. I want to be able to pull new versions if
    // they're buggy and un-trigger the update check!
    if (!then || latestVersion.release.major==0 || (now-then)>86400000)
    {
        // cancel any update request in progress - there damn well shouldn't be
        // one, if everything's working!
        if (externalXmlRequest != null)
        {
            externalXmlRequest.abort();
            externalXmlRequest = null;
        }
        // set up the request
        externalXmlRequest = new XMLHttpRequest();
        // when we've got the data, this calls processVersionResponse()
        // to handle it, passing the data to it
        externalXmlRequest.onload = function(e) {processVersionResponse(e, externalXmlRequest);}
        // gives the url to query, amongst other things; if you want to see
        // how the other end works, visit that URL in your browser and
        // view the source
        externalXmlRequest.open("GET", "http://www.phantomgorilla.com/update/versions.php?product=BBCRadio&version="+versionDisplayString(currentVersion)+"&protocol=1", true);
        // make sure we don't get a cached version of the page
        externalXmlRequest.setRequestHeader("Cache-Control", "no-cache");
        // don't send anything
        externalXmlRequest.send(null);
    }
    determineUpdateAvailability();
}


// searches the nodes in the page for the one that relates to this product, then
// gets the data from it - the latest version number :)
function processVersionResponse(e, request)
{
    // set the externalXmlRequest to null, so we know no request is in progress
    externalXmlRequest = null;
    // if there's an answer with text
    if (request.responseText && request.status && request.status==200)
    {
        // We got a response, so let's note this time as the most recent check
        then = (new Date).getTime();
        if (window.widget) widget.setPreferenceForKey(then,"lastupdatecheck");
        
        if (debug) alert(request.responseText);
        var response = JSON.parse(request.responseText);
        var release = response.products.BBCRadio.release;
        var beta = response.products.BBCRadio.beta;
        if (release.version) {
            latestVersion.release = splitVersion(release.version);
            if (release.url) {
                updateUrl = release.url;
            } else {
                updateUrl = null;
            }
        }
        if (currentVersion.beta && beta.version) {
            latestVersion.betaRelease = splitVersion(beta.version);
            if (beta.url) {
                updateUrl = beta.url;
            } else {
                updateUrl = null;
            }
        }
        // stores the latest version information we just got as a preference;
        // even though latestVersion is a global variable, we save this now so
        // that on next load (when it will have cleared) we'll still know what
        // the latest version is without checking online (till a day is up)
        widget.setPreferenceForKey(versionDisplayString(latestVersion.release), "latestversion");
        widget.setPreferenceForKey(versionDisplayString(latestVersion.betaRelease), "LatestBeta");
        widget.setPreferenceForKey(updateUrl, "UpdateUrl");
    }
    determineUpdateAvailability();
}


// does some stuff depending on whether an update is available
function determineUpdateAvailability() {
    // now we've got the latest version information, we have to check to see if
    // there's actually an update, or if this version is current.
    // if the latest version available is greater than the current version:
    if (checkIfVersionHigher(currentVersion, latestVersion.release)) {
        // hide the Phantom Gorilla graphic in the lower left
        document.getElementById("phantom").style.display="none";
        // show the "update" graphics instead
        document.getElementById("update").style.display="block";
        document.getElementById("front_update").style.display="block";
        if (debug) alert("*** Update found! ***");
    }
    else if (currentVersion.beta && checkIfVersionHigher(currentVersion, latestVersion.betaRelease)) {
        // hide the Phantom Gorilla graphic in the lower left
        document.getElementById("phantom").style.display="none";
        // show the "update" graphics instead
        document.getElementById("update").style.display="block";
        document.getElementById("front_update").style.display="block";
        if (debug) alert("*** Update found! ***");
    }
    // otherwise, do the reverse (in case the update availability has reverted
    // while the widget is open; unlikely, but possible
    else {
        document.getElementById("update").style.display="none";
        document.getElementById("front_update").style.display="none";
        document.getElementById("phantom").style.display="block";
        if (debug) alert("No update found");
    }
}


function openUpdatePage() {
    var the_url;
    if (updateUrl && updateUrl.length > 7) {
        the_url = updateUrl;
    } else {
        the_url = "http://www.phantomgorilla.com/widgets/";
    }
    if (window.widget) {
        widget.openURL(the_url);
    } else {
		window.location.href = the_url;
    }
}


// Originally changestation(), but it evolved into a behemoth prefs function.
// setPrefs() is called whenever a station is chosen from the popup menu.
// It saves the location of the image and the stream url of the corresponding
// station and then calls functions which change the image and the audio stream
// on the front of the widget.
function setPrefs()
{
    var elem = document.getElementById("station");
    // look up the station in the stationList object.
	var selectedOption=elem.options[elem.selectedIndex].value;
    widget.setPreferenceForKey(selectedOption,"CurrentStation");
    widget.setPreferenceForKey(selectedOption,makeKey("CurrentStation"));
    currentStation = stationList[selectedOption];
    
    changeImage();
} 


// changeImage() is called on loading and after preference changes.
// It checks which images should be shown and changes the html to show it.
// Also adds any additional info such as local radio area over the image,
// if appropriate.
function changeImage()
{
    // naughty check for hidden key "UseOfficialLogos"
    var useOfficial = widget.preferenceForKey("UseOfficialLogos");
    if (useOfficial) {
        useOfficial = true;
        document.getElementById("main").className = "official";
    } else {
        useOfficial = false;
        document.getElementById("main").className = "";
    }
    
    var logo = currentStation.logo;
    // check for valid preference
    if (logo && (logo.length>0)) {
        // set the source of the image correctly
        if (useOfficial) {
        // more naughtiness; switch to official logos if hidden key told us to
            document.getElementById("logo").src = "images/official/"+logo;
        } else {
        // use normal logos otherwise
            document.getElementById("logo").src = "images/normal/"+logo;
        }
    }
    // if this is local radio
    if (currentStation.coverage == "local") {   
        var localContainer = document.getElementById("local_name_container");
        // display the name of the local station on the front
        if (currentStation.shortname) {
            localContainer.innerText = currentStation.shortname;
        } else {
            localContainer.innerText = currentStation.name;
        }
        localContainer.style.display="block";  
        // and hide the LW/FM indicator as it's redundant here
        document.getElementById("extrainfo_container").style.display="none";
    }
    // otherwise, if there are multiple station bands/variants
    else if (currentStation.extra && currentStation.extra.length > 0) {
        var extraInfo = document.getElementById("extrainfo_container");
        // show FM/LW info
        extraInfo.innerText = currentStation.extra;
        extraInfo.style.display="block";
        // hide the local info, it's redundant
        document.getElementById("local_name_container").style.display="none";
    }
    else {
        // turn off both if neither needed
        document.getElementById("local_name_container").style.display="none";
        document.getElementById("extrainfo_container").style.display="none";
    }
}


// changeStream() - formerly changeStreamUrl() - is called when the play button
// is pressed, or after preferences are changed. It checks which stream should
// be playing, and then composes some html which embeds the real player plugin
// in the widget to stream the audio. Then the html is pasted into the widget.
function changeStream()
{    
    // if we want Flash and it's available
    // or there's no RealPlayer link
    if (UseFlash && currentStation.flash) {
        placeFlash(currentStation.flash, true);
    } else {
        // check that prefs are valid
        if (currentStation.realplayer && (currentStation.realplayer.length > 0))
        {
            // this section composes the html in the form an embed tag for
            // RealPlayer, which is provided with a custom source url which we read
            // from preferences earlier
            var html = '<embed src="'+currentStation.realplayer+'" type="audio/x-pn-realaudio-plugin" autostart="true" height="0" width="0" console="one" nojava="true" id="rp" scriptcallbacks="OnPlayStateChange,OnBuffering" />';
            // change the inner html of the radio span to the html we just composed
            document.getElementById("radio").innerHTML = html;
            // set the player to the volume the slider is showing
            setTimeout("changeVolume();", 1000);
            // ensures that RealPlayer plugin suppresses any errors, avoiding ugly
            // dialogue boxes
            document.getElementById("rp").SetWantErrors(true);
        }
    }
}



/*****************************************/
// HANDLING PLAYER INTERFACE MOUSE EVENTS
/*****************************************/


// called when the mouse clicks on certain objects.
// we get passed the reference of the object on which the mouse went down,
// and an english reference for it
function theMouseDownHandler(event, div, clickedElement)
{
    // loads each image we could use into a cariable, so that they're cached
    // and won't flicker when we swap them in
	var img = new Image;
	img.src = "images/play2_clicked.png";
	img.src = "images/play2.png";
	img.src = "images/pause2_clicked.png";
	img.src = "images/pause2.png";
	img.src = "images/knob_clicked.png";
	img.src = "images/knob.png";

    // passes the clickedElement to the global variable
    // wasClickedElement so the other functions can use it
    wasClickedElement=clickedElement;

    // if we clicked on the play button
    if (wasClickedElement=="play")
    {
        // change to the clicked play button image
        document.getElementById("play").src="images/play2_clicked.png";
    }
    // if we clicked on the pause button
    if (wasClickedElement=="pause")
    {
        // change to the clicked play button image
        document.getElementById("pause").src="images/pause2_clicked.png";
    }
    // if it's the volume slider/knob that was clicked
    else if (wasClickedElement=="knob" || wasClickedElement=="slider")
    {
        if (UseFlash) return;
        // change to the clicked knob image
        document.getElementById("knob").src="images/knob_clicked.png";
        // Save starting positions of cursor and element.
        cursorStartX = event.clientX;
        knobStartLeft = parseInt(document.getElementById("knob").style.left, 10)
	}
    // if it's the volume slider, move the knob to point of click
    if (wasClickedElement=="slider")
    {
        // allowing for the size of the knob (we want it centred over our click,
        // but we position it using the left hand side) move it all the way to
        // the left if user clicked on the left boundary (global, set at start)
        if ((cursorStartX-6) < leftBound)
        {
            var skipKnobToHere = leftBound;
        }
        // same as above, but for right boundary (also set at start)
        else if ((cursorStartX-6) > rightBound)
        {
            var skipKnobToHere = rightBound;
        }
        // otherwise, skip the knob to where we clicked; again, allowing for the
        // size & the fact that we want to centre the knob over our click
        else
        {
            var skipKnobToHere = (cursorStartX-6);
        }
        // do the actual moving via setting the CSS property
        document.getElementById("knob").style.left = skipKnobToHere + "px";
        // reset the knob's starting position to the new position, in case
        // we do some dragging after the click
        // Are you impressed that this is allowed? ;) I'm quite proud of this!
        knobStartLeft = parseInt(document.getElementById("knob").style.left, 10)
        // calls the function to change the volume, based on the new position
        changeVolume();
	}
	// add listeners, to trigger handlers if the mouse does anything
	document.addEventListener("mousemove", theMouseMoveHandler, true);
	document.addEventListener("mouseup", theMouseUpHandler, true);
	div.addEventListener("mouseover", theMouseOverHandler, true);
	div.addEventListener("mouseout", theMouseOutHandler, true);
	// set it up so we know globally that the mouse is down, and where
	div.arrowInside = true;
	theCurrentMouseDownButton = div;
    // I dunno, but Apple do this in their button handlers (from which all these
    // are derived)
	event.stopPropagation();
	event.preventDefault();
}


// when the mouse moves after mousedown, this is called
function theMouseMoveHandler (event)
{
    // if we clicked on the knob or the slider; ie, we're in a drag
    if (wasClickedElement=="knob" || wasClickedElement=="slider")
    {
        if (UseFlash) return;
        // calculate the knob position based on its start point and how much
        // the cursor has moved
        var newPosition = (knobStartLeft + event.clientX - cursorStartX);
        // if that's within the slider boundaries, move there
        if (newPosition > leftBound && newPosition < rightBound)
        {
            document.getElementById("knob").style.left = newPosition + "px";
        }
        // if it's beyond the left boundary (set globally, at start)
        else if (newPosition < leftBound)
        {
            // just skip to the left boundary instead
            document.getElementById("knob").style.left = leftBound + "px";
        }
        // if it's beyond the right boundary (again, set globally, at start)
        else if (newPosition > rightBound)
        {
            // skip to the right boundary
            document.getElementById("knob").style.left = rightBound + "px";
        }
        // calls changeVolume() to update the volume based on these moves
        changeVolume();
	}
    // like I said, Apple use these
	event.stopPropagation();
	event.preventDefault();
}


// this is called if the mouse goes back over the element - obviously, we must
// already have had a mouseout event for this to be being called!
function theMouseOverHandler (event)
{
    // if the mouse button's still down, and the original click was on the button
	// (not the knob or slider, we don't let the knob change
    // back from blue till the mouse button is released, so there's no point
    // in calling them here again)
	if (theCurrentMouseDownButton && wasClickedElement=="play")
	{
        // change to the clicked images
        document.getElementById("play").src="images/play2_clicked.png";
	}
	else if (theCurrentMouseDownButton && wasClickedElement=="pause")
	{
        // change to the clicked images
        document.getElementById("pause").src="images/pause2_clicked.png";
	}
	// set variable to show the mouse is currently over the element clicked on
	theCurrentMouseDownButton.arrowInside = true;
    // getting bored of saying "I don't know"
	event.stopPropagation();
	event.preventDefault();
}


// called if the mouse moves away from the element clicked on
function theMouseOutHandler (event)
{
	// if original click was on play button
	if (theCurrentMouseDownButton && wasClickedElement=="play")
	{
        // switch to the unclicked image, as the mouse isn't over it
        document.getElementById("play").src="images/play2.png";
	}
	// and the pause button
	else if (theCurrentMouseDownButton && wasClickedElement=="pause")
	{
        // switch to the unclicked image, as the mouse isn't over it
        document.getElementById("pause").src="images/pause2.png";
	}
    // set variable to indicate the mouse is outside the element clicked on
	theCurrentMouseDownButton.arrowInside = false;
	// je ne sais pas
	event.stopPropagation();
	event.preventDefault();
}


// when the button is released, this is called - the business part of all this
function theMouseUpHandler (event)
{
	if (theCurrentMouseDownButton && wasClickedElement=="play")
	{
        // put the unclicked images back, the mouse is up!
        document.getElementById("play").src="images/play2.png";
	}
	else if (theCurrentMouseDownButton && wasClickedElement=="pause")
	{
        // put the unclicked images back, the mouse is up!
        document.getElementById("pause").src="images/pause2.png";
	}
	else if (theCurrentMouseDownButton && (wasClickedElement=="knob" || wasClickedElement=="slider"))
	{
        if (UseFlash) return;
        // put the unclicked images back, the mouse is up!
        document.getElementById("knob").src="images/knob.png";
	}
	// callback to the client, clean up
	document.removeEventListener("mousemove", theMouseMoveHandler, true);
	document.removeEventListener("mouseup", theMouseUpHandler, true);
	if (theCurrentMouseDownButton)
	{
		theCurrentMouseDownButton.removeEventListener("mouseover", theMouseOverHandler, true);
		theCurrentMouseDownButton.removeEventListener("mouseout", theMouseOutHandler, true);
	}
    // Quintus est puer Romanus
	event.stopPropagation();
	event.preventDefault();
    // IF the button was depressed over an element and it's now been
	// released *over the same element*
	if (theCurrentMouseDownButton &&
	    theCurrentMouseDownButton.arrowInside)
	{
    // if it was the play button clicked, call a function
	if (wasClickedElement=="play")
        {
            // this places the plugin (unless audio is already playing)
            playClicked(event);
        }
    // if it was the pause button instead, call another function
	else if (wasClickedElement=="pause")
        {
            // this removes the plugin
            pauseClicked();
        }
    }
	// reset the variable, as the button's been released
	theCurrentMouseDownButton = null;
}


// openWhatsOnPage() tells the widget to open the appropriate schedule in
// a browser window
function openWhatsOnPage()
{
    // read the preference for the nickname of the station - a component
    // of the url we use that's specific to each station.
    var nickname = currentStation.nickname;
    // check for a pref
    if (nickname && (nickname.length > 0))
    {
        // compose
        var schedulePage = "http://www.bbc.co.uk/"+nickname+"/programmes/schedules";
        // tweak for weird stations
        if (nickname=="radio4") schedulePage += currentStation.extra.toLowerCase();
        if (nickname=="radioscotland") schedulePage += "/fm";
        if (nickname=="radiowales") schedulePage += "/fm";
        if (nickname=="radiocymru") schedulePage = "http://www.bbc.co.uk/radiocymru/amserlen/";
        // open the link
        widget.openURL(schedulePage);
    }
}


// called when the play button is clicked (from theMouseUphandler), places the
// plugin
function playClicked(e)
{
    // we can disable tests with a plist key, or case-by-case with alt.
    // combine both possibilities for ease of use.
    if ((e && e.altKey) || disableTests) {
        var noTests = true;
    } else {
        var noTests = false;
    }
    
    // double check for Flash use
    // necessary now we have multiple instances
    checkForFlash();

    // hide any status messages showing
    if (document.getElementById("status").innerHTML!="") {
        clearTimeout(statusTimeout);
        statusTimeout=null;
        hideStatus();
    }
    // clear flash timer details
    if (flashTimer.trigger) {
        clearTimeout(flashTimer.trigger);
        flashTimer.trigger = null;
        flashTimer.seconds = 0;
    }
    // start the progress indicator running (so long as it's 
    // currently stopped, otherwise we'd make the animation
    // restart and it'd look horrible)
    if (progressIndicatorTimer==null)
    {
        startProgressIndicator('large');
    }
    // hide the play button, show the pause button
    document.getElementById("play").style.display="none";
    document.getElementById("pause").style.display="block";
    // skip if tests disabled
    if (noTests) {
        changeStream();
    } else {
        // pinched this idea from the BBC ListenLive widget; hope Duncan Ponting
        // doesn't mind. It's a great trick!
        // sniff for the Real Player plugin; first set up a flag as false
        var pluginHere = false;
        // loop through all the plugins
        for (var i=0;i < navigator.plugins.length; i++)
        {
            // check for an appropriately named plugin
            var pluginName = (UseFlash ? "flash" : "realplayer");
            if (navigator.plugins[i].name.toLowerCase().indexOf(pluginName)!=(-1))
            {
                //  set our flag to true
                pluginHere = true;
                // break the loop, we've found it
                break;
            }
        }
        // if the plugin's not installed
        if (!pluginHere)
        {
            // show a status message, with link
            if (UseFlash) showStatus(e,'Flash plugin not found.<br>Click this text to download<br>it now. (You must also<br>close this widget.)',20,function(){javascript:widget.openURL("http://get.adobe.com/flashplayer/")});
            if (!UseFlash) showStatus(e,'RealPlayer plugin not found. Click this text to download,<br>then run it. (You must also<br>close this widget.)',20,function(){widget.openURL("http://www.real.com/R/RC.021705realhome_1_2_2_1_1_3.ecomm...R/realguide.real.com/r/https_.html?url=order.real.com%2fpt%2forder.html%3fppath%3dcpmacpl060204a%26country%3dUS%26language%3dEN%26opage%3drealhome%26src%3d021705realhome_1_2_2_1_1_3")});
            // pretend pause is clicked, so we stop the indicator and reset the
            // button images
            pauseClicked();
        }
        // if the plugin's installed
        // use xmlhttprequest to test for internet connection - if we can complete
        // a connection to the BBC radio player page we proceed and place the player
        // (if we place it without a connection we could get a crash later!)
        else
        {
            // set up the request
            xmlhttp = new XMLHttpRequest();
            // location et al
            xmlhttp.open("HEAD", "http://www.bbc.co.uk/radio/",true);
            //xmlhttp.open("HEAD", "http://google.com",true);
            // every time the state of the request changes
            // run the following code
            xmlhttp.onreadystatechange = processTestResponse;
            // send nothing
            xmlhttp.send(null);
            processTestResponse();
        }
    }
}


// Processes the state changes of the connection test. Ugly workaround for
// Safari 4 beta's poor handling of XMLHttpRequests when there's no internet
// connection – onreadystatechange doesn't progress properly so we explicitly
// check.
function processTestResponse()
{
    if (xmlhttp && xmlhttp.readyState && xmlhttp.readyState==1) {
        // timeout test is explained more thoroughly in schedule.js
        if (testInProg && testInProg!=null) {
            clearTimeout(testInProg);
            testInProg = null;
            showStatus(null,"You don’t appear to be connected to the internet. Please check your connection and try again.", 10);
            clearTimeout(statusTimeout);
            statusTimeout = setTimeout('showStatus(null,"If you’re certain you’re connected, option-click<br>(⌥-click) the Play button<br>to override this test.", 7)',10000);
            pauseClicked();
        } else {
            testInProg = setTimeout("processTestResponse();", 10000);
        }
    }
    // if the connection request is complete
    else if (xmlhttp && xmlhttp.readyState && xmlhttp.readyState==4)
    {
        //alert("st: "+xmlhttp.status+" "+xmlhttp.statusText);
        clearTimeout(testInProg);
        testInProg = null;
        // and if the request was successful
        if (xmlhttp.status==200)
        {
            // place an instance of the plugin, change the volume,
            // change the play button to a pause button
            changeStream();
        }
        // if unsuccessful
        else
        {
            // calls a function to show a "no connection" warning
            // really ugly hack to stop this appearing when the user cancels
            if (document.getElementById("pause").style.display=="block") {
                showStatus(null,"You don’t appear to be connected to the internet. Please check your connection and try again.", 10);
            }
            // pretend pause is clicked, so we stop the indicator and
            // reset the button images
            pauseClicked();
        }
    }
}


// Called when pause is clicked, to remove the plugin and stop audio
function pauseClicked()
{
	// stop the progress indicator, in case it was running
    stopProgressIndicator();
    // resets the inner html of the span holding the plugin
	// and resets the flash timeout (otherwise if we hide
    // Dashboard too soon the flash placement audio loss
    // workaround might cause audio to start again)
    clearInterval(flashTimer.trigger);
    flashTimer.seconds = 0;
    flashTimer.trigger = null;
    document.getElementById("radio").innerHTML="";
    // hide the pause button, shaow the play button
    document.getElementById("pause").style.display = "none";
    document.getElementById("play").style.display = "block";
    // abandon connection test timeout
    if (testInProg && testInProg!=null) {
        clearTimeout(testInProg);
        testInProg = null;
    }
	// stop the xmlhttprequest - on slow connections it might not have
	// completed yet and it can lead to audio starting after pause has
	// been clicked.
	if (xmlhttp!=null)
	{
		xmlhttp.abort();
		xmlhttp = null;
    }
    // double check for Flash use
    // necessary now we have multiple instances
    checkForFlash();
}


// show a status message
function showStatus(e, message, duration, clickhandler)
{
    if (!clickhandler) var clickhandler = function () { return;};
    var statusContainer = document.getElementById("status");
	// stop any status fade event that's due to be called
	clearTimeout(statusTimeout);
	statusTimeout = null;
    // pass the event to a global variable (we can't pass it on to hideStatus()
    // because we call it via setTimeout(), which doesn't support that, so we
    // have to make it global)
    globalStatusEvent=e;
    // set the inner html to the message we got passed
    statusContainer.innerHTML=message;
    // set the onclick
    statusContainer.onclick= clickhandler;
    // show the status div (although it's transparent at the moment)
    statusContainer.style.display="block";
    // setting up an AppleAnimator, to fade in the status message (speedily)
    // starting from 0 opacity
    var from=0;
    // if it's running
    if(null != statusAnimator)
    {
        // stop
        statusAnimator.stop();
        // start from current value
        from = statusAnimator.animations[0].now;
    }
    // if the shift key is down, the multiplier is 10, if not, 1
    if (!e || e==null) { var multiplier = 1 } else {
        var multiplier = e.shiftKey ? 10 : 1;
    }
    // upshot of this is, increasing values get passed to statusFadeHandler
    statusAnimator = new AppleAnimator(500*multiplier, 13, from, 1, statusFadeHandler);
    // starts it all off!
    statusAnimator.start();
    // call hideStatus after the amount of time we specified, to give plenty of
    // time to read it (we multiply by 1000 as 'duration' is specified in
    // seconds, but setTimeout uses thousanths of a second)
    duration = (duration*1000);
    statusTimeout = setTimeout('hideStatus()',duration);
}


// hides the status div, called after 3 seconds from the showStatus() function
function hideStatus()
{
    // setting up an AppleAnimator, to fade in the status message speedily
    // starting from 1 opacity
    var from = (document.getElementById("status").style.opacity ? document.getElementById("status").style.opacity : 1);
    // if it's running
    if(null != statusAnimator)
    {
        // stop
        statusAnimator.stop();
        // start from current value
        from = statusAnimator.animations[0].now;
    }
    // if the shift key is down, the multiplier is 10, if not, 1
    if (!globalStatusEvent || globalStatusEvent==null) { var multiplier = 1 } else {
        var multiplier = globalStatusEvent.shiftKey ? 10 : 1;
    }
    // upshot of this is, decreasing values get passed to statusFadeHandler
    statusAnimator = new AppleAnimator(500*multiplier, 13, from, 0, statusFadeHandler);
    // starts it all off!
    statusAnimator.start();
}


// handles the fading in and out of the status div
function statusFadeHandler(animation, current, from, to)
{
    // set the current opacity to what the Animator passes it
    document.getElementById("status").style.opacity=current;
    if (current==0)
    {
        // hide the status div (although it's transparent now)
        document.getElementById("status").style.display="none";
        // reset message to nothing
        document.getElementById("status").innerHTML="";
    }
}


// changes the volume when called (when placing a new plugin or when changing
// the slider position), based on where the slider knob is
function changeVolume()
{
    clearTimeout(volumeCounter);
    // get the knob position
    var sliderPosition = parseInt(document.getElementById("knob").style.left, 10);
    // the start of the slider is several pixels in from the left, so deduct those;
    // also the slider lingth is 33, so times by 3 to get a value out of 99
    // (so close!) - RealPlayer sets volume as a percentage, you see
    var newVolume = (sliderPosition-leftBound)*3
    // if there's a plugin instance currently placed
    if (document.getElementById("rp"))
    {
        // tell it to set the volume to the new value
        document.getElementById("rp").SetVolume(newVolume);
    }
    volumeCounter = setTimeout("saveVolume("+newVolume+");",500);
}


function saveVolume(vol) {
    // if we're in Dashboard
    if (window.widget) {
        // save this as a preference (we'll use it to determine where the slider
        // should be on next launch)
        widget.setPreferenceForKey(vol,"Volume");
        widget.setPreferenceForKey(vol,makeKey("Volume"));
    }
}


// called at launch, to position the slider according to the volume level we've
// got saved in preferences
function restoreVolume()
{
    // if we're in Dashboard
    if (window.widget)
    {
        // retrieve the preference for this instance
        var newVolume = widget.preferenceForKey(makeKey("Volume"));
        // if it doesn't exist or make sense, load the global setting and overwrite it
        if (!newVolume || newVolume.length <= 0) {
            newVolume = widget.preferenceForKey("Volume");
            if (!newVolume || newVolume.length <= 0) {
                newVolume = 99;
                widget.setPreferenceForKey(newVolume,"Volume");
            }
            widget.setPreferenceForKey(newVolume,makeKey("Volume"));
        }
        if (newVolume) {
            // divide by 3 to get a value out of 33 (the slider length); then
            // add leftBound to this (the slider offset from the left margin) so we get
            // an absolute position for the knob
            var sliderPosition = (newVolume/3)+leftBound;
            // set the knob to this position
            document.getElementById("knob").style.left = sliderPosition;
        }
    }
}



/****************************/
// CALLBACK FROM THE PLUGIN
/****************************/


// called when the plugin changes state - eg. starts playing, or stops; is
// passed the old and new states.
function OnPlayStateChange(oldState,newState)
{
    // if the plugin starts playing
    if (newState==3)
    {
        // stops the indicator
        stopProgressIndicator();
    }
    // otherwise, if the player stops or pauses (accidentally)
    else if (newState==0 || newState==4)
    {
        // pause then play - "seamless" autoresume
        if (debug) alert("Playback interrupted; attempting autoresume.");
        pauseClicked();
        playClicked();
    }
}


// Called whenever plugin buffering begins - is passed the type of buffering
// (startup/network congestion etc), and the percent complete.
function OnBuffering(flag,percent)
{
	// start the progress indicator, for visual feedback
	startProgressIndicator('large');
}



/*********************************/
// HIDING AND SHOWING PREFERENCES
/*********************************/


// showPrefs() is called when the preferences flipper is clicked upon.  It
// freezes the front of the widget, hides the front div, unhides the back div,
// and then flips the widget over.
function showPrefs(event)
{
	// stop the schedule from periodically refreshing - we can't see it!
	clearTimeout(scheduleTimer);
	// check if the drawer is open, and close it if it is
	if (drawerStatus=="open")
	{
		if (event && event.altKey) {
            goingToAdvPrefsWithDrawerOpen = true;
        } else {
            goingToPrefsWithDrawerOpen = true;
        }
        showHideDrawer(event);
	} else {
        // get the front and back divs
        var front = document.getElementById("front");
        var back = document.getElementById("back");
        // if alt key, go to the other prefs page (flash/real)
        if ((event && event.altKey) || goingToAdvPrefsWithDrawerOpen) {
            back = document.getElementById("back2");
        }
        // if we're in Dashboard
        if (window.widget) {
            // double check for Flash use
            // necessary now we have multiple instances
            if (document.getElementById("radio").innerHTML=="") checkForFlash();
            // remember which station is selected now
            // so we only interrupt audio later if we've changed station
            preferenceBefore = currentStation.name;
            // freezes the widget so that you can change it without the user noticing
            widget.prepareForTransition("ToBack");
        }
        // hide the front, show the back
        front.style.display="none";
        back.style.display="block";
        // and flip the widget over
        if (window.widget)
        {  setTimeout ('widget.performTransition();', 0);  }
	}
}


// hidePrefs() is called by the done button on the back side of the widget. It
// performs the opposite transition as showPrefs() does.
function hidePrefs()
{
    // get the front and back
	var front = document.getElementById("front");
	var back = document.getElementById("back");
    // if a new station has been selected, we change the audio stream playing;
    // otherwise we leave it alone so that playback isn't interrupted
    if (currentStation.name != preferenceBefore)
    {
        checkForFlash();
        // blank out the radio playback as soon as possible
        document.getElementById("radio").innerHTML = "";
        // pretend we clicked the play button, to start the audio
        playClicked(event);
		// get the new schedule
		getSchedule();
    }
	else
	{
	// refresh the current schedule
		refreshSchedule();
	}
	// freezes the widget and prepares it for the flip back to the front
    widget.prepareForTransition("ToFront");
	// changes the image on the schedule drawer button
	updateDrawerButton();
	// hide the back
	back.style.display="none";
	// show the front
	front.style.display="block";
	// and flip the widget back to the front
	if (window.widget)
	{	setTimeout ('widget.performTransition();', 0);   }
	// if the drawer was open
	if (goingToPrefsWithDrawerOpen)
	{
		goingToPrefsWithDrawerOpen = false;
		setTimeout ('showHideDrawer();', 1000);
	}
}

// hidePrefs() is called by the done button on the back side of the widget. It
// performs the opposite transition as showPrefs() does.
function hidePrefs2()
{
    // get the front and back
	var front = document.getElementById("front");
	var back = document.getElementById("back2");

    // get checkbox state, set the UseFlash key, and then
    // check if we should be using Flash for this station.
    var temp_UseFlash = (document.getElementById("radiobutton2").getElementsByTagName("input")[0].checked ? true : false);
    var oldFlash = UseFlash;
    widget.setPreferenceForKey(temp_UseFlash,"UseFlash");
    checkForFlash();
    // if flash usage for station changed restart playback with new preference
    if (oldFlash!=UseFlash) {
        document.getElementById("radio").innerHTML = "";
        // pretend we clicked the play button, to start the audio
        playClicked(event);
        // get the new schedule
        refreshSchedule();
    }

	// freezes the widget and prepares it for the flip back to the front
    if (window.widget) widget.prepareForTransition("ToFront");
	// changes the image on the schedule drawer button
	updateDrawerButton();
	// hide the back
	back.style.display="none";
	// show the front
	front.style.display="block";
	// and flip the widget back to the front
	if (window.widget)
	{	setTimeout ('widget.performTransition();', 0);   }
	// if the drawer was open
	if (goingToAdvPrefsWithDrawerOpen)
	{
		goingToAdvPrefsWithDrawerOpen = false;
		setTimeout ('showHideDrawer();', 1000);
	}
}


function checkForFlash() {
    var flashHere = false;
    var rpHere = false;
    for (var i=0;i < navigator.plugins.length; i++)
    {
        if (navigator.plugins[i].name.toLowerCase().indexOf("flash")!=(-1)) {
            flashHere = true;
        }
        if (navigator.plugins[i].name.toLowerCase().indexOf("realplayer")!=(-1)) {
            rpHere = true;
        }
    }
    // check for hidden key "UseFlash"
    var temp_UseFlash = widget.preferenceForKey("UseFlash");

    if (temp_UseFlash!==false) {
        temp_UseFlash = true;
    }
    
    document.getElementById("radiobutton2").getElementsByTagName("input")[0].checked = temp_UseFlash;
    document.getElementById("radiobutton1").getElementsByTagName("input")[0].checked = !temp_UseFlash;

    if ((temp_UseFlash || !currentStation.realplayer || (flashHere && !rpHere)) && currentStation.flash) {
        UseFlash = true;
        document.body.className = "flash";
        document.getElementById("volume").title="Sorry, you can only change volume at the BBC iPlayer website. ⌥-click (option-click) to go to iPlayer now, then stop and start playback for your change to take effect.";
    } else {
        UseFlash = false;
        document.body.className = "";
        document.getElementById("volume").title="";
    }
}


function placeFlash(url, autoplay) {
    if (autoplay!=true) autoplay = false;
    var html = '<div id="semp-wrapper" class="semp-wrapper"><embed type="application/futuresplash" src="http://www.bbc.co.uk/emp/10player.swf?revision=18182_18346" width="0" height="0" id="bbc_emp_embed_semp-wrapper" name="bbc_emp_embed_semp-wrapper" bgcolor="#000000" quality="high" wmode="default" allowfullscreen="false" allowscriptaccess="always" flashvars="config=http://www.bbc.co.uk/emp/iplayer/config.xml&amp;config_settings_showPopoutButton=false&amp;config_settings_autoPlay='+autoplay+'&amp;playlist='+url+'&amp;config_settings_showFullScreenButton=false"></div>';
    document.getElementById("radio").innerHTML = html;
	// reset timer etc
    flashTimer.seconds = 0;
    clearInterval(flashTimer.trigger);
    flashTimer.trigger = null;
    // run timer
    flashTimer.trigger = setInterval("flashCounter();", 1000);
}

function flashCounter() {
    flashTimer.seconds += 1;
    // after 10 seconds we assume audio has started
    if (flashTimer.seconds==10) {
        stopProgressIndicator();
        clearTimeout(flashTimer.trigger);
        flashTimer.trigger = null;
        flashTimer.seconds = 0;
    }
}

// whenever radio button for flash / realplayer is clicked
function radioClicked(target) {
    if (target.id=="radiobutton1") {
        document.getElementById("radiobutton1").getElementsByTagName("input")[0].checked = true;
        document.getElementById("radiobutton2").getElementsByTagName("input")[0].checked = false;
    } else {
        document.getElementById("radiobutton2").getElementsByTagName("input")[0].checked = true;
        document.getElementById("radiobutton1").getElementsByTagName("input")[0].checked = false;
    }
}


// decides if we should use the hack, based on OS version
// (Tiger doesn't need it) and the plist key "UseRPHack".
// Key takes priority if set.
function checkForHackUse() {
    // check for hidden key "UseRPHack"
    var temp_UseHack = widget.preferenceForKey("UseRPHack");

    if (temp_UseHack==false || temp_UseHack==0 || temp_UseHack=="false" || temp_UseHack=="no") {
        useHack = false;
    }
    else if (temp_UseHack==true || temp_UseHack==1 || temp_UseHack=="true" || temp_UseHack=="yes") {
        useHack = true;
    }
    
    // only if no valid key is found
    else {
        // user agent check
        var ua = navigator.userAgent;

        // if we find a 10.4 indication, don't use hack
        // unlikely as dashboard's webkit doesn't provide an os
        // version but it's worth a try
        if (ua.indexOf("Mac OS X 10_4")!=-1) {
            useHack = false;
        }
        // likewise if we find 10.5, it's true. This is expected!
        else if (ua.indexOf("Mac OS X 10_5")!=-1) {
            useHack = true;
        }
        // otherwise fall back on detecting webkit at or under 419
        // (version in 10.4's dashboard).
        else {
            var wkVer = ua.match(/AppleWebKit\/([0-9]|\.)+/);
            if (wkVer && wkVer.length >= 1) wkVer = wkVer[0];
            wkVer = wkVer.match(/([0-9]|\.)+/)
            if (wkVer && wkVer.length >= 1) wkVer = wkVer[0].split(".");
            wkVer = parseInt(wkVer[0]);

            if (wkVer && !isNaN(wkVer) && wkVer<=419) {
                useHack = false;
            } else {
                useHack = true;
            }
        }
    }
}


function checkForTestDisable() {
    // check for hidden key "DisableTests"
    var temp_DisableTests = widget.preferenceForKey("DisableTests");

    if (temp_DisableTests==true || temp_DisableTests==1 || temp_DisableTests=="true" || temp_DisableTests=="yes") {
        disableTests = true;
    }
}


// formats a version object for display
function versionDisplayString(version) {
    return version.major+"."+(version.minor<10 ? "0"+version.minor : version.minor)+(version.beta ? "b"+version.beta : "");
}


// doUpgrade is called whenever the last used version is
// older than the current version, in order that we can ditch
// old preferences and do other cleanup activities.
function doUpgrade(lastUsedVersion) {
    alert("Upgrading BBC Radio Widget from "+(lastUsedVersion ? "v"+versionDisplayString(lastUsedVersion) : "[unknown version]")+" to v"+versionDisplayString(currentVersion)+".");
    var stationPopup = document.getElementById("station");
    var optionToSelect = widget.preferenceForKey("selection");

    // if the last used version is under 4.1b1
    if (checkIfVersionHigher(lastUsedVersion,splitVersion("4.1b1"))) {
        if(debug) alert("Hi! The v4.1b1 upgrade code is running.");
        widget.setPreferenceForKey(currentStation.nickname,"CurrentStation");
        widget.setPreferenceForKey(currentStation.nickname,makeKey("CurrentStation"));
        widget.setPreferenceForKey(null,"logo");
        widget.setPreferenceForKey(null,"nickname");
        widget.setPreferenceForKey(null,"selection");
        widget.setPreferenceForKey(null,"url");
        widget.setPreferenceForKey(null,"area");
        widget.setPreferenceForKey(null,"FlashUrl");
        widget.setPreferenceForKey(null,"extraInfo");
        widget.setPreferenceForKey(null,"last_used_version");
        widget.setPreferenceForKey(null,"channel_id");
    }

    // set this version as last used version
    widget.setPreferenceForKey(versionDisplayString(currentVersion),"LastUsedVersion");

    // helps us know if the widget has been cleanly upgraded
    widget.setPreferenceForKey(true,"WaitingForRestart");
    showStatus(null,'Widget upgraded! Please choose "Keep", and reload (&#x2318;R) or click this message to restart Dashboard.',30,function(){widget.system("killall Dock; sleep 3; [[ -e /Applications/Dashboard.app ]] && open /Applications/Dashboard.app", killDashboardHandler);});
    justShownRestartMessage = true;
}

function killDashboardHandler () {
    return;
}

function makeKey(key) {
    return widget.identifier + "-" + key;
}

function goToiPlayer(event)
{
    if (UseFlash && event && event.altKey) {
        if (window.widget) widget.openURL(currentStation.flash.replace(/playlist/, "playlive"));
    }
}
