// xmlhttprequest object
var http = null;
// arrays for cached times; used for refresh
var startTime = null;
var endTime = null;
// remember last item selected in schedule
var lastItem = null;
// timer for schedule refresh
var scheduleTimer = null;
// timer for "http" xmlhttprequest that allows us to time it out
var schedInProg = null;
var schedType;

// grabs the raw schedule from the API or web page
function getSchedule() {
    resetScheduleArea();
    // if the station has an API key
	if (currentStation.api) {
        getApi();
	} else {
        getScrape();
    }
}

// clear schedule and put up a "loading" message
function resetScheduleArea() {
    // if a request is in progress, kill it and clear timers
    if (http!=null) {
        http.abort();
        http=null;
        if (schedInProg && schedInProg!=null) {
            clearTimeout(schedInProg);
            schedInProg = null;
        }
    }
    // post a loading message
    htmlToPaste = "<div style=\"color:rgb(115, 115, 115);padding:35px 10px 0;text-align:center;font-size:10px;\">Loading...</div>";
    // refresh scroll area because of loading message
    document.getElementById("scrollArea").object.refresh();
    document.getElementById("content").innerHTML = htmlToPaste;
    // reset stores because this could be a new station
    startTime = null;
    endTime = null;
}

// get api data
function getApi() {
    schedType = "api";
    // get morning and midnight, to use as the bounds for our request
    var thisMorning = calculateThisMorning();
    var midnight = calculateMidnight();
    var apiurl = "http://www0.rdthdo.bbc.co.uk/cgi-perl/api/query.pl?method=bbc.schedule.getProgrammes&channel_id="+currentStation.api+"&start="+thisMorning+"&end="+midnight+"&limit=100&detail=schedule";
    if (debug) alert("Schedule request (api), \""+apiurl+"\"");
    // arrange and send request
    http = new XMLHttpRequest();
    http.open("GET", apiurl, true);
    http.onreadystatechange = processResponse;
    http.send(null);
    processResponse();
}

// get scraped page data
function getScrape() {
    schedType = "scrape";
    // test to check we can process the scrape properly
    // otherwise, why bother?
    if (document.getElementsByClassName==undefined || document.getElementsByTagName==undefined) {
        scheduleError("Sorry, your Mac OS version can't fetch this station's schedule.");
        return false;
    }
    var nickname = currentStation.nickname;
    // compose
    var schedulePage = "http://www.bbc.co.uk/"+nickname+"/programmes/schedules";
    // various special cases
    if (nickname=="radio4") schedulePage += "/lw";
    if (nickname=="radioscotland") schedulePage += "/fm";
    if (nickname=="radiowales") schedulePage += "/fm";
    if (nickname=="radiocymru") schedulePage = "http://www.bbc.co.uk/radiocymru/amserlen/";
    // arrange and send the request
    if (debug) alert("Schedule request (scrape), \""+schedulePage+"\"");
    http = new XMLHttpRequest();
    http.open("GET", schedulePage, true);
    http.onreadystatechange = processResponse;
    http.send(null);
    processResponse();
}

// deals with the schedule request state changes, and despatch
// the result appropriately. type is "api" or "scrape"
function processResponse() {
    // clear timer, so we don't ready any more
    // requests while this one is happening
    clearTimeout(scheduleTimer);
    scheduleTimer = null;
    // safari 4 beta can let requests get stuck in state 1 if
    // there's no connection, so arrange a timeout
    if (http && http.readyState && http.readyState == 1)
    {
        if (schedInProg && schedInProg!=null) {
            // this will be the second runthrough on 1 (called by
            // setTimout) so we're giving up here. Clear timers,
            // abort, and display the error.
            clearTimeout(schedInProg);
            schedInProg = null;
            if (debug) alert("Schedule request timed out");
            scheduleError();
            http.abort();
            http = null;
        } else {
            // first run through - try again in 7 seconds
            schedInProg = setTimeout("processResponse();",7000);
        }
    }
    else if (http && http.readyState && http.readyState == 4)
	{
        if (debug) alert("Schedule request returned status: "+http.status);
        // request completed, so clear out timeout
        clearTimeout(schedInProg);
        schedInProg = null;
        // if it's successful, despatch to appropriate
        // processor for preparation and schedule insertion...
		if (http.status == 200)	{
            if (schedType=="api") {
                processApi();
            } else {
                processScrape();
            }
		}
        // ...otherwise, show an error
		else { scheduleError(); }
        // refresh scroll area, it's got new information in it
		document.getElementById("scrollArea").object.refresh();
	}
}

// This parses the xml returned by the BBC API, and
// formats it for use in the schedule drawer
function processApi() {
    programmes = http.responseXML.getElementsByTagName("programme");
    var htmlToPaste = "";
    // these arrays used for cached refresh of current programme
    startTime = new Array(programmes.length);
    endTime = new Array(programmes.length);
    for (var i = 0; i < programmes.length; i++) {
        var title = programmes[i].getAttribute("title");
        var synopsis = getElementText("synopsis", programmes[i], 0);
        var scrapedDate = getElementText("start", programmes[i], 0);
        startTime[i] = new Date();
        startTime[i].setISO8601(scrapedDate);
        var duration = getElementText("duration", programmes[i], 0);
        // duration is provided in hours:mins:seconds, so it's
        // easiest to add them all on separately
        duration = duration.split(":");
        duration[0] = duration[0]*1 + startTime[i].getHours();
        duration[1] = duration[1]*1 + startTime[i].getMinutes();
        duration[2] = duration[2]*1 + startTime[i].getSeconds();
        endTime[i] = new Date();
        // set end from start date, and set time
        // from our duration calculations
        endTime[i].setDate(startTime[i].getDate());
        endTime[i].setHours(duration[0], duration[1], duration[2]);
        // compose item html
        htmlToPaste += composeItemHtml(i, title, synopsis, startTime[i], endTime[i]);
    }
    // reset our "last item selected" variable -
    // it's currently meaningless as we just rewrote the schedule
    lastItem = null;
    // paste html
    document.getElementById("content").innerHTML = htmlToPaste;
    // refresh schedule to highlight current item
    refreshSchedule(null, true);
}

function processScrape() {
    // find our hidden div and paste into it,
    // so we can traverse the response
    var holder = document.getElementById("sched_temp");
    holder.innerHTML = http.responseText;
    // trawl through the html to find the info we need
    // fail gracefully if it's unexpected
    var content = document.getElementById("broadcasts");
        if (!content) {
            scheduleError();
            return false;
        }
    var programmes = content.getElementsByClassName("clearfix");
        if (!programmes || programmes.length < 1) {
            scheduleError(); return false;
        }
    if (debug) alert(programmes.length + " programmes found");
    // only grab the programmes supplied with UTC dates
    startTime = new Array(programmes.length);
    timedProgs = new Array();
    for (var i=0; i<programmes.length; i++) {
        if (programmes[i].id) {
            // strips the leading "t"
            if (programmes[i].id.charAt(0)=="t") programmes[i].id = programmes[i].id.substr(1);
            // adds to array of programmes with start times
            timedProgs.push(programmes[i]);
        }
    }
    // make start & end time arrays
    // used for cached refresh of current programme
    startTime = new Array(timedProgs.length);
    endTime = new Array(timedProgs.length);
    var htmlToPaste = "";
    for (var i=0; i<timedProgs.length; i++) {
        startTime[i] = new Date();
        // set object from supplied UTC date
        startTime[i].setISO8601(timedProgs[i].id);
        // grab times
        var start = timedProgs[i].getElementsByClassName("starttime");
        var end = timedProgs[i].getElementsByClassName("endtime");
            if (start && start.length >= 1) start = start[0].innerHTML;
            if (end && end.length >= 1) end = end[0].innerHTML;
        // strip any other characters
        start = start.match(/([0-1][0-9]|[2][0-3]):([0-5][0-9])/);
        end = end.match(/([0-1][0-9]|[2][0-3]):([0-5][0-9])/);
            if (start && start.length >= 1) start = start[0];
            if (end && end.length >= 1) end = end[0];
        // calculate duration in minutes
        // had some weirdness using parseInt()
        start = (start.substr(0,2)*60 + start.substr(3,2)*1);
        end = (end.substr(0,2)*60 + end.substr(3,2)*1);
        var duration;
        if (start > end) {
            // deals with programmes spanning midnight
            duration = (1440-start)+end;
        } else {
            duration = (end - start);
        }
        // now create date object end time using start + duration
        var newMins = startTime[i].getMinutes() + duration;
        endTime[i] = new Date();
        // set end equal to start, then add on the right number of minutes
        endTime[i].setTime(startTime[i].getTime());
        endTime[i].setHours(startTime[i].getHours(), startTime[i].getMinutes(), 0);
        endTime[i].setMinutes(newMins);
        //alert("start "+startTime[i]+" end "+endTime[i]);
        // get title and description
        var title = timedProgs[i].getElementsByClassName("title");
            if (title && title.length >= 1) title = title[0].innerHTML;
        var synopsis = timedProgs[i].getElementsByClassName("description");
            if (synopsis && synopsis.length >= 1) synopsis = synopsis[0].innerHTML;
        // strip whitespace
        title = title.replace(/^\s+|\s+$/g,"");
        synopsis = synopsis.replace(/^\s+|\s+$/g,"");
        // create html for this item, and add it in
        htmlToPaste += composeItemHtml(i, title, synopsis, startTime[i], endTime[i]);
    }
    // get rid of the schedule html
    holder.innerHTML = "";
    // reset our "last item selected" variable -
    // it's currently meaningless as we just rewrote the schedule
    lastItem = null;
    // paste html
    document.getElementById("content").innerHTML = htmlToPaste;
    // refresh schedule to highlight current item
    refreshSchedule(null, true);
}

// creates html for each item, for collation and insertion
function composeItemHtml(index, title, synopsis, startDate, endDate) {
    // cleanup to remove html tags (mainly for scrape's benefit)
    title = title.replace(/<(.|\n)*?>/g, "");
    synopsis = synopsis.replace(/<(.|\n)*?>/g, "");
    // compose item's html
    var theHtml = "<div title=\"" + title;
    theHtml += ": " + synopsis + "\"";
    theHtml += " class=\"item\" id=\"item" + index + "\">";
    theHtml += "<span class=\"time\">" + (formatDateForDisplay(startDate)) + "</span>";
    theHtml += title + "</div>";
    // return our html
    return theHtml;
}

// called when something goes wrong - shows a message, and
// sets up a second attempt later
function scheduleError(customMessage)
{
    // fallback to scrape if we're trying the API
    if (schedType == "api") {
        resetScheduleArea();
        getScrape();
        return;
    }
    if (!customMessage) customMessage = "Sorry, the schedule<br>information could not<br>be retrieved.";
    if (debug) alert("Schedule error handler fired, \""+customMessage.replace(/<br>/gi," ")+"\"");
    // paste error into drawer
    htmlToPaste = "<div style=\"color:rgb(115, 115, 115);padding:20px 0 0 0;text-align:center;font-size:10px;\">"+customMessage+"<br><a style=\"font-size: 9px; display: block; margin: 6px auto 0;\" href=\"javascript:openWhatsOnPage();\">Open page at bbc.co.uk &raquo;</a></div>";
    document.getElementById("content").innerHTML = htmlToPaste;
    // reset caches
    startTime = null;
    endTime = null;
    // try in 2 minutes
	clearTimeout(scheduleTimer);
    scheduleTimer = setTimeout('refreshSchedule()',120000);
    // refresh scroll area, because it's got new content
    document.getElementById("scrollArea").object.refresh();
}

// refreshes, matching the current item and highlighting it,
// from cache or asking for fresh info if needed; scroll
// can be "forceScroll" to reveal the programme found
// if stopIfNoMatch is true, don't try to fetch schedule if
// we can't match a programme; avoids infinite loop
function refreshSchedule(scroll, stopIfNoMatch)
{
    if (debug) alert("Looking for current programme match");
    // clear timeout for this function - we're doing it now!
	clearTimeout(scheduleTimer);
    // current time, used to find current programme
	var rightNow = new Date();
	var gotMatch = false;
    // if we've got caches
	if ((startTime!=null)&&(endTime!=null))
	{
        // check each pair of start/end times, testing to see
        // if the current time lies between them
		for (var i = 0; i < startTime.length; i++) {
			if ((rightNow >= startTime[i]) && (rightNow < endTime[i]))
			{
                // only change things if this isn't the same item as last time
				if (i!=lastItem) {
                    // change this item to be highlighted, and
                    // unhighlight the previous selection
					if (lastItem != null) { document.getElementById("item"+lastItem).className = "item"; }
					document.getElementById("item"+i).className = "currentitem";
					lastItem = i;
                    // programme has changed, so we'll want to reveal it
					scroll = "forceScroll";
                    if (debug) alert("Found new programme, start " +startTime[i]+ " / end " +endTime[i]+ ", now "+rightNow);
				} else if (debug) {
                    alert("Current programme still matches (start " +startTime[i]+ " / end " +endTime[i]+ ", now "+rightNow+")");
                }
                // reveal the programme; scroll to just above the item,
                // or the top if it's close.
				if (scroll=="forceScroll") {
					if (i>2) {
                        document.getElementById("scrollArea").object.verticalScrollTo((i-2)*13+1);
                    }
					else {
                        document.getElementById("scrollArea").object.verticalScrollTo("0");
                    }
				}
				gotMatch = true;
			}
		}
	}
    // do this again in 30 seconds
	scheduleTimer = setTimeout('refreshSchedule()',30000);

    // hack to improve no-scrollbar display (hides it completely if
    // there's only one screen of content)
    // timeout gives this a better chance of catching it after scrollbar has updated
    setTimeout('if (document.getElementById("scrollArea").object._scrollbars[0].hidden) { document.getElementById("content").className = "hidden"; } else { document.getElementById("content").className = ""; }',0);

    if (gotMatch == false) {
        if (debug) alert("No programme found");
        // no match, and we've tried getting new content, so throw
        // an error and avoid an infinite loop
        if (stopIfNoMatch==true) {
            if (debug) alert("Avoided infinite loop");
			scheduleError();
        // try getting new content and finding a match there
        } else {
            if (!schedInProg || schedInProg==null) getSchedule();
        }
    }
}

// utility function for grabbing info from api response
function getElementText(tag, parentElem, index) {
    var result = "";
    result = parentElem.getElementsByTagName(tag)[index];
    if (result)
    {
        // get text, accounting for possible
        // whitespace (carriage return) text nodes
        if (result.childNodes.length > 1) {
            return result.childNodes[1].nodeValue;
        } else {
            return result.firstChild.nodeValue;
        }
    } else {
        return "n/a";
    }
}

// utility function to grab 1 sec past midnight
function calculateThisMorning() {
    var today = new Date();
    today.setHours("0", "0", "1");
    var theUTCDate = today.toISO8601String("5");
    return theUTCDate;
}

// utility function to grab 1 sec before midnight tonight
function calculateMidnight() {
    var today = new Date();
    today.setHours("23", "59", "59");
    var theUTCDate = today.toISO8601String("5");
    return theUTCDate;
}

// utility function to format the date with leading zeroes
function formatDateForDisplay(date) {
    var displayDate = "";
    if (date.getHours()<10) {
        displayDate = "0"+date.getHours()+":"
    } else {
        displayDate = date.getHours()+":"
    }
    if (date.getMinutes()<10) {
        displayDate = displayDate+"0"+date.getMinutes()
    } else {
        displayDate = displayDate+date.getMinutes()
    }
	return displayDate;
}

// utility function to set a js date object from a UTC date
Date.prototype.setISO8601 = function (string) {
    var regexp = "([0-9]{4})(-([0-9]{2})(-([0-9]{2})" +
        "(T([0-9]{2}):([0-9]{2})(:([0-9]{2})(\.([0-9]+))?)?" +
        "(Z|(([-+])([0-9]{2}):([0-9]{2})))?)?)?)?";
    var d = string.match(new RegExp(regexp));

    var offset = 0;
    var date = new Date(d[1], 0, 2);
    if (d[13]!="Z") {
        if (d[3]) { date.setMonth(d[3] - 1); }
        if (d[5]) { date.setDate(d[5]); }
        if (d[7]) { date.setHours(d[7]); }
        if (d[8]) { date.setMinutes(d[8]); }
        if (d[10]) { date.setSeconds(d[10]); }
        if (d[12]) { date.setMilliseconds(Number("0." + d[12]) * 1000); }
        if (d[14]) {
            offset = (Number(d[16]) * 60) + Number(d[17]);
            offset *= ((d[15] == '-') ? 1 : -1);
        }
        offset -= date.getTimezoneOffset();
        time = (Number(date) + (offset * 60 * 1000));
    } else {
        if (d[3]) { date.setUTCMonth(d[3] - 1); }
        if (d[5]) { date.setUTCDate(d[5]); }
        if (d[7]) { date.setUTCHours(d[7]); }
        if (d[8]) { date.setUTCMinutes(d[8]); }
        if (d[10]) { date.setUTCSeconds(d[10]); }
        if (d[12]) { date.setUTCMilliseconds(Number("0." + d[12]) * 1000); }
        time = (Number(date));
    }

    this.setTime(Number(time));
}

// utility function to make a UTC date
Date.prototype.toISO8601String = function (format, offset) {
    /* accepted values for the format [1-6]:
     1 Year:
       YYYY (eg 1997)
     2 Year and month:
       YYYY-MM (eg 1997-07)
     3 Complete date:
       YYYY-MM-DD (eg 1997-07-16)
     4 Complete date plus hours and minutes:
       YYYY-MM-DDThh:mmTZD (eg 1997-07-16T19:20+01:00)
     5 Complete date plus hours, minutes and seconds:
       YYYY-MM-DDThh:mm:ssTZD (eg 1997-07-16T19:20:30+01:00)
     6 Complete date plus hours, minutes, seconds and a decimal
       fraction of a second
       YYYY-MM-DDThh:mm:ss.sTZD (eg 1997-07-16T19:20:30.45+01:00)
    */
    if (!format) { var format = 6; }
    if (!offset) {
        var offset = 'Z';
        var date = this;
    } else {
        var d = offset.match(/([-+])([0-9]{2}):([0-9]{2})/);
        var offsetnum = (Number(d[2]) * 60) + Number(d[3]);
        offsetnum *= ((d[1] == '-') ? -1 : 1);
        var date = new Date(Number(Number(this) + (offsetnum * 60000)));
    }

    var zeropad = function (num) { return ((num < 10) ? '0' : '') + num; }

    var str = "";
    str += date.getUTCFullYear();
    if (format > 1) { str += "-" + zeropad(date.getUTCMonth() + 1); }
    if (format > 2) { str += "-" + zeropad(date.getUTCDate()); }
    if (format > 3) {
        str += "T" + zeropad(date.getUTCHours()) +
               ":" + zeropad(date.getUTCMinutes());
    }
    if (format > 5) {
        var secs = Number(date.getUTCSeconds() + "." +
                   ((date.getUTCMilliseconds() < 100) ? '0' : '') +
                   zeropad(date.getUTCMilliseconds()));
        str += ":" + zeropad(secs);
    } else if (format > 4) { str += ":" + zeropad(date.getUTCSeconds()); }

    if (format > 3) { str += offset; }
    return str;
}